==> toycars-0.3.0.tar.gz: toycars-0.3.0/src/Scoobie/ScParticles.cpp <==
/*
	Copyright (C) 2005 by Ruben Zilibowitz <rzilibowitz@users.sourceforge.net>
	Part of the Toy Cars Project http://toycars.sourceforge.net

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License.
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY.

	See the COPYING file for more details.
*/

/*
 *  ScParticles.cpp
 *  Scoobie
 *
 *  Created by Ruben Zilibowitz on 13/01/05.
 *
 */

#include <cassert>
#include "ScParticles.h"
#include "ScException.h"

//
// constructor
//
ScParticles::ScParticles(long inMaxParticles, float inRed, float inGreen, float inBlue)
	:	maxParticles(inMaxParticles),
		lowestUnusedElement(0),
		lastElementInUse(0),
		numberOfParticlesActive(0),
		red(inRed), green(inGreen), blue(inBlue)
{
	particleArray = new Particle [maxParticles];
	if (particleArray == NULL)
		ScThrowErr("Failed to allocate memory for particleArray");
	
	for (long curParticle = 0; curParticle < maxParticles; curParticle++)
	{
		particleArray[curParticle].life = -1.0;
	}
}

//
// destructor
//
ScParticles::~ScParticles()
{
	if (particleArray != NULL)
		delete [] particleArray;
}

//
// draw - overrides super class ScComponent function draw
//
void ScParticles::draw()
{
	long		index;
	Particle*	particlePtr;
	
	// set to brown color
	glColor3f(red, green, blue);
	
	glDisable(GL_TEXTURE_2D);
	glEnable(GL_TEXTURE_1D);
	glBegin(GL_POINTS);
	
	for (index = 0, particlePtr = particleArray; index <= lastElementInUse; index++, particlePtr++)
	{
		if (particlePtr->life > 0.0)
		{
			glVertex2d(particlePtr->loc_x, particlePtr->loc_y);
		}
	}
	
	glEnd();
	glDisable(GL_TEXTURE_1D);
	glEnable(GL_TEXTURE_2D);
	
	// call super class draw
	ScComponent::draw();
}

//
// clear
//
void ScParticles::reset()
{
	for (long curParticle = 0; curParticle < maxParticles; curParticle++)
	{
		particleArray[curParticle].life = -1.0;
	}
	
	lowestUnusedElement = 0;
	lastElementInUse = 0;
	numberOfParticlesActive = 0;
}

//
// newParticle
//
void ScParticles::newParticle(const Particle &p)
{
		// Add a particle only if there is room.
	if (numberOfParticlesActive < maxParticles)
	{
			// Find the lowest unused array element
			// (Note: this assumes there *is* an available element.)
		while (particleArray[lowestUnusedElement].life > 0.0)
			lowestUnusedElement++;
		
		assert(lowestUnusedElement < maxParticles);
		
			// assign the particle to the free place
		particleArray[lowestUnusedElement] = p;
		
			// If the new particle is higher than lastElementInUse, extend it.
		if (lowestUnusedElement > lastElementInUse)
			lastElementInUse = lowestUnusedElement;
	
		numberOfParticlesActive++;
		lowestUnusedElement++;
	}
}

//
// process
//
void ScParticles::process(ScFloat dt)
{
	Particle*	particlePtr;
	long		index;
	
	assert(lastElementInUse < maxParticles);
	for (index = 0, particlePtr = particleArray; index <= lastElementInUse; index++, particlePtr++)
	{
		if (particlePtr->life > 0.0)
		{
			particlePtr->life -= dt;
			
			if (particlePtr->life <= 0.0)
			{
				numberOfParticlesActive--;
				
					// If this dead particle opened up a space, update lowestUnusedElement.
				if (index < lowestUnusedElement)
					lowestUnusedElement = index;
			}
			else
			{
				// calculate new position
				particlePtr->step(dt);
			}
		}
	}
	
	if (numberOfParticlesActive <= 0)
	{
		lastElementInUse = 0;
	}
	else
	{
			// See if the lastElementInUse isn't in use any more.
			// If not, find the next element that still is.
		while (particleArray[lastElementInUse].life <= 0.0 && lastElementInUse > 0)
			lastElementInUse--;
	}
}

==> toycars-0.3.3.tar.gz: toycars-0.3.3/src/Scoobie/ScParticles.cpp <==
/*
	Copyright (C) 2005 by Ruben Henner Zilibowitz <rzilibowitz@users.sourceforge.net>
	Part of the Toy Cars Project http://toycars.sourceforge.net

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License.
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY.

	See the COPYING file for more details.
*/

/*
 *  ScParticles.cpp
 *  Scoobie
 *
 *  Created by Ruben Henner Zilibowitz on 13/01/05.
 *
 */

#include <cassert>
#include "ScParticles.h"
#include "ScException.h"

//
// constructor
//
ScParticles::ScParticles(long inMaxParticles, float inRed, float inGreen, float inBlue)
	:	maxParticles(inMaxParticles),
		lowestUnusedElement(0),
		lastElementInUse(0),
		numberOfParticlesActive(0),
		red(inRed), green(inGreen), blue(inBlue)
{
	particleArray = new Particle [maxParticles];
	if (particleArray == NULL)
		ScThrowErr("Failed to allocate memory for particleArray");
	
	for (long curParticle = 0; curParticle < maxParticles; curParticle++)
	{
		particleArray[curParticle].life = -1.0;
	}
}

//
// destructor
//
ScParticles::~ScParticles()
{
	if (particleArray != NULL)
		delete [] particleArray;
}

//
// draw - overrides super class ScComponent function draw
//
void ScParticles::draw()
{
	long		index;
	Particle*	particlePtr;
	
	// set to brown color
	glColor3f(red, green, blue);
	
	glDisable(GL_TEXTURE_2D);
	glEnable(GL_TEXTURE_1D);
	glBegin(GL_POINTS);
	
	for (index = 0, particlePtr = particleArray; index <= lastElementInUse; index++, particlePtr++)
	{
		if (particlePtr->life > 0.0)
		{
			glVertex2d(particlePtr->loc_x, particlePtr->loc_y);
		}
	}
	
	glEnd();
	glDisable(GL_TEXTURE_1D);
	glEnable(GL_TEXTURE_2D);
	
	// call super class draw
	ScComponent::draw();
}

//
// clear
//
void ScParticles::reset()
{
	for (long curParticle = 0; curParticle < maxParticles; curParticle++)
	{
		particleArray[curParticle].life = -1.0;
	}
	
	lowestUnusedElement = 0;
	lastElementInUse = 0;
	numberOfParticlesActive = 0;
}

//
// newParticle
//
void ScParticles::newParticle(const Particle &p)
{
		// Add a particle only if there is room.
	if (numberOfParticlesActive < maxParticles)
	{
			// Find the lowest unused array element
			// (Note: this assumes there *is* an available element.)
		while (particleArray[lowestUnusedElement].life > 0.0)
			lowestUnusedElement++;
		
		assert(lowestUnusedElement < maxParticles);
		
			// assign the particle to the free place
		particleArray[lowestUnusedElement] = p;
		
			// If the new particle is higher than lastElementInUse, extend it.
		if (lowestUnusedElement > lastElementInUse)
			lastElementInUse = lowestUnusedElement;
	
		numberOfParticlesActive++;
		lowestUnusedElement++;
	}
}

//
// process
//
void ScParticles::process(ScFloat dt)
{
	Particle*	particlePtr;
	long		index;
	
	assert(lastElementInUse < maxParticles);
	for (index = 0, particlePtr = particleArray; index <= lastElementInUse; index++, particlePtr++)
	{
		if (particlePtr->life > 0.0)
		{
			particlePtr->life -= dt;
			
			if (particlePtr->life <= 0.0)
			{
				numberOfParticlesActive--;
				
					// If this dead particle opened up a space, update lowestUnusedElement.
				if (index < lowestUnusedElement)
					lowestUnusedElement = index;
			}
			else
			{
				// calculate new position
				particlePtr->step(dt);
			}
		}
	}
	
	if (numberOfParticlesActive <= 0)
	{
		lastElementInUse = 0;
	}
	else
	{
			// See if the lastElementInUse isn't in use any more.
			// If not, find the next element that still is.
		while (particleArray[lastElementInUse].life <= 0.0 && lastElementInUse > 0)
			lastElementInUse--;
	}
}
